@@ -21,6 +21,7 @@ from static_frame.core.display import DisplayConfig
 from static_frame.core.display import DisplayHeader
 from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import ErrorInitIndex
+from static_frame.core.exception import ErrorInitIndexNonUnique
 from static_frame.core.exception import LocEmpty
 from static_frame.core.exception import LocInvalid
 from static_frame.core.index_base import IndexBase
@@ -355,12 +356,18 @@ class Index(IndexBase):
         if hasattr(labels, '__len__'): # not a generator, not an array
             # resolving the dtype is expensive, pass if possible
             if len(labels) == 0: #type: ignore
-                labels = EMPTY_ARRAY
+                if dtype is None:
+                    labels = EMPTY_ARRAY
+                else:
+                    labels = np.empty(0, dtype=dtype)
             else:
                 labels, _ = iterable_to_array_1d(labels, dtype=dtype)
         else: # labels may be an expired generator, must use the mapping
             if len(mapping) == 0: #type: ignore
-                labels = EMPTY_ARRAY
+                if dtype is None:
+                    labels = EMPTY_ARRAY
+                else:
+                    labels = np.empty(0, dtype=dtype)
             else:
                 labels, _ = iterable_to_array_1d(mapping, dtype=dtype) #type: ignore
         # all arrays are immutable
@@ -464,7 +471,9 @@ class Index(IndexBase):
                 except ValueError: # Automap will raise ValueError of non-unique values are encountered
                     pass
                 if self._map is None:
-                    raise ErrorInitIndex(f'labels ({len(tuple(labels))}) have non-unique values ({len(set(labels))})')
+                    raise ErrorInitIndexNonUnique(
+                            f'labels ({len(tuple(labels))}) have non-unique values ({len(set(labels))})'
+                            )
                 size = len(self._map)
             else: # must assume labels are unique
                 # labels must not be a generator, but we assume that internal clients that provided loc_is_iloc will not give a generator
@@ -618,6 +627,56 @@ class Index(IndexBase):
             self._update_array_cache()
         return bool(self._labels.size)
 
+    #---------------------------------------------------------------------------
+    # set operations
+
+    def _ufunc_set(self: I,
+            func: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray],
+            other: tp.Union['IndexBase', 'Series']
+            ) -> I:
+        '''
+        Utility function for preparing and collecting values for Indices to produce a new Index.
+        '''
+        if self._recache:
+            self._update_array_cache()
+
+        if self.equals(other, compare_dtype=True):
+            # compare dtype as result should be resolved, even if values are the same
+            if (func is self.__class__._UFUNC_INTERSECTION or
+                    func is self.__class__._UFUNC_UNION):
+                # NOTE: this will delegate name attr
+                return self if self.STATIC else self.copy()
+            elif func is self.__class__._UFUNC_DIFFERENCE:
+                if self._DTYPE is None: #type: ignore
+                    # an index with a variable dtype accepts a dtype argument
+                    return self.__class__((), dtype=self.dtype) #type: ignore
+                # if self._DTYPE is defined, the default constructor does not take a dtype argument
+                return self.__class__(())
+
+        if isinstance(other, np.ndarray):
+            operand = other
+            assume_unique = False
+        elif isinstance(other, IndexBase):
+            operand = other.values
+            assume_unique = True # can always assume unique
+        elif isinstance(other, ContainerOperand):
+            operand = other.values
+            assume_unique = False
+        else:
+            raise NotImplementedError(f'no support for {other}')
+
+        cls = self.__class__
+
+        # using assume_unique will permit retaining order when operands are identical
+        labels = func(self.values, operand, assume_unique=assume_unique) # type: ignore
+
+        if id(labels) == id(self.values):
+            # NOTE: favor using cls constructor here as it permits maximal sharing of static resources and the underlying dictionary
+            return cls(self)
+
+        return cls.from_labels(labels)
+
+
     #---------------------------------------------------------------------------
     def _drop_iloc(self, key: GetItemKeyType) -> 'IndexBase':
         '''Create a new index after removing the values specified by the loc key.
@@ -1076,7 +1135,7 @@ class Index(IndexBase):
                     & isna_array(other.values, include_none=False))
             eq[isna_both] = True
 
-        if not eq.all():
+        if not eq.all(): # avoid returning a NumPy Bool
             return False
         return True
 
@@ -1115,6 +1174,32 @@ class Index(IndexBase):
             values.flags.writeable = False
         return self.__class__(values, name=self._name)
 
+    @doc_inject(selector='fillna')
+    def fillna(self, value: tp.Any) -> 'Index':
+        '''Return an :obj:`Index` with replacing null (NaN or None) with the supplied value.
+
+        Args:
+            {value}
+        '''
+        values = self.values # force usage of property for cache update
+        sel = isna_array(values)
+        if not np.any(sel):
+            return self if self.STATIC else self.copy()
+
+        value_dtype = np.array(value).dtype
+        assignable_dtype = resolve_dtype(value_dtype, values.dtype)
+
+        if values.dtype == assignable_dtype:
+            assigned = values.copy()
+        else:
+            assigned = values.astype(assignable_dtype)
+
+        assigned[sel] = value
+        assigned.flags.writeable = False
+
+        return self.__class__(assigned, name=self._name)
+
+
     #---------------------------------------------------------------------------
     # export
 
@@ -1125,11 +1210,16 @@ class Index(IndexBase):
         from static_frame import Series
         return Series(self.values, name=self._name)
 
+
     def add_level(self, level: tp.Hashable) -> 'IndexHierarchy':
-        '''Return an IndexHierarhcy with an added root level.
+        '''Return an IndexHierarchy with an added root level.
         '''
         from static_frame import IndexHierarchy
-        return IndexHierarchy.from_tree({level: self.values})
+        from static_frame import IndexHierarchyGO
+
+        cls = IndexHierarchy if self.STATIC else IndexHierarchyGO
+        return cls.from_tree({level: self.values})
+
 
     def to_pandas(self) -> 'pandas.Index':
         '''Return a Pandas Index.
