@@ -132,6 +132,10 @@ class IndexBase(ContainerOperand):
         raise NotImplementedError() #pragma: no cover
 
 
+    @property
+    def ndim(self) -> int:
+        raise NotImplementedError()
+
     #---------------------------------------------------------------------------
     # constructors
 
@@ -153,10 +157,19 @@ class IndexBase(ContainerOperand):
 
         if isinstance(value, pandas.MultiIndex):
             # iterating over a hierarchucal index will iterate over labels
-            name = tuple(value.names)
+            name: tp.Optional[tp.Tuple[tp.Hashable, ...]] = tuple(value.names)
+            # if not assigned Pandas returns None for all components, which will raise issue if trying to unset this index.
+            if all(n is None for n in name): #type: ignore
+                name = None
+            depth = value.nlevels
+
             if not cls.STATIC:
-                return IndexHierarchyGO.from_labels(value, name=name)
-            return IndexHierarchy.from_labels(value, name=name)
+                return IndexHierarchyGO.from_labels(value,
+                        name=name,
+                        depth_reference=depth)
+            return IndexHierarchy.from_labels(value,
+                    name=name,
+                    depth_reference=depth)
         elif isinstance(value, pandas.DatetimeIndex):
             # if IndexDatetime, use cls, else use IndexNanosecond
             if issubclass(cls, IndexDatetime):
@@ -232,34 +245,7 @@ class IndexBase(ContainerOperand):
             func: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray],
             other: tp.Union['IndexBase', 'Series']
             ) -> I:
-        '''
-        Utility function for preparing and collecting values for Indices to produce a new Index.
-        '''
-        if self._recache:
-            self._update_array_cache()
-
-        if isinstance(other, np.ndarray):
-            opperand = other
-            assume_unique = False
-        elif isinstance(other, IndexBase):
-            opperand = other.values
-            assume_unique = True # can always assume unique
-        elif isinstance(other, ContainerOperand):
-            opperand = other.values
-            assume_unique = False
-        else:
-            raise NotImplementedError(f'no support for {other}')
-
-        cls = self.__class__
-
-        # using assume_unique will permit retaining order when opperands are identical
-        labels = func(self.values, opperand, assume_unique=assume_unique) # type: ignore
-
-        if id(labels) == id(self.values):
-            # NOTE: favor using cls constructor here as it permits maximal sharing of static resources and the underlying dictionary
-            return cls(self)
-        return cls.from_labels(labels)
-
+        raise NotImplementedError()
 
     def intersection(self: I, other: tp.Union['IndexBase', 'Series']) -> I:
         '''
@@ -286,7 +272,6 @@ class IndexBase(ContainerOperand):
                 self.__class__._UFUNC_DIFFERENCE,
                 other)
 
-
     #---------------------------------------------------------------------------
     # metaclass-applied functions
 
