@@ -5,42 +5,33 @@ import numpy as np
 
 from automap import AutoMap  # pylint: disable = E0611
 
-# from static_frame.core.util import DTYPE_DATETIME_KIND
 
+from static_frame.core.doc_str import doc_inject
+from static_frame.core.index import _INDEX_GO_SLOTS
+from static_frame.core.index import _INDEX_SLOTS
+from static_frame.core.index import _IndexGOMixin
+from static_frame.core.index import Index
+from static_frame.core.index import IndexGO
+from static_frame.core.util import DateInitializer
+from static_frame.core.util import DT64_DAY
+from static_frame.core.util import DT64_H
+from static_frame.core.util import DT64_M
+from static_frame.core.util import DT64_MONTH
+from static_frame.core.util import DT64_MS
+from static_frame.core.util import DT64_NS
+from static_frame.core.util import DT64_S
+from static_frame.core.util import DT64_US
+from static_frame.core.util import DT64_YEAR
 from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import IndexInitializer
-# from static_frame.core.util import mloc
 from static_frame.core.util import key_to_datetime_key
-
-from static_frame.core.util import DateInitializer
-from static_frame.core.util import YearMonthInitializer
-from static_frame.core.util import YearInitializer
-
-from static_frame.core.util import to_datetime64
-from static_frame.core.util import to_timedelta64
-
-from static_frame.core.util import DT64_YEAR
-from static_frame.core.util import DT64_MONTH
-from static_frame.core.util import DT64_DAY
-from static_frame.core.util import DT64_M
-from static_frame.core.util import DT64_H
-from static_frame.core.util import DT64_S
-from static_frame.core.util import DT64_MS
-from static_frame.core.util import DT64_US
-from static_frame.core.util import DT64_NS
-
 from static_frame.core.util import TD64_DAY
 from static_frame.core.util import TD64_MONTH
 from static_frame.core.util import TD64_YEAR
-
-from static_frame.core.index import _IndexGOMixin
-from static_frame.core.index import _INDEX_SLOTS
-from static_frame.core.index import _INDEX_GO_SLOTS
-
-from static_frame.core.index import Index
-from static_frame.core.index import IndexGO
-
-from static_frame.core.doc_str import doc_inject
+from static_frame.core.util import to_datetime64
+from static_frame.core.util import to_timedelta64
+from static_frame.core.util import YearInitializer
+from static_frame.core.util import YearMonthInitializer
 
 
 if tp.TYPE_CHECKING:
@@ -109,7 +100,9 @@ class IndexDatetime(Index):
 
     def loc_to_iloc(self,  # type: ignore
             key: GetItemKeyType,
+            *,
             offset: tp.Optional[int] = None,
+            partial_selection: bool = False,
             ) -> GetItemKeyType:
         '''
         Specialized for IndexData indices to convert string data representations into np.datetime64 objects as appropriate.
@@ -118,7 +111,9 @@ class IndexDatetime(Index):
         return Index.loc_to_iloc(self,
                 key=key,
                 offset=offset,
-                key_transform=key_to_datetime_key)
+                key_transform=key_to_datetime_key,
+                partial_selection=partial_selection,
+                )
 
     #---------------------------------------------------------------------------
     def to_pandas(self) -> 'pandas.DatetimeIndex':
