@@ -29,6 +29,7 @@ from static_frame.core.util import DTYPE_BOOL
 from static_frame.core.util import KEY_ITERABLE_TYPES
 from static_frame.core.util import KEY_MULTIPLE_TYPES
 from static_frame.core.util import resolve_dtype_iter
+from static_frame.core.util import EMPTY_TUPLE
 
 
 # if tp.TYPE_CHECKING:
@@ -98,15 +99,16 @@ class IndexLevel:
 
     @classmethod
     def from_tree(cls,
-            tree: tp.Any, # recersively defined
+            tree: tp.Any, # recursively defined
             index_constructors: tp.Optional[IndexConstructors] = None,
+            depth_reference: tp.Optional[int] = None,
             ) -> 'IndexLevel':
         '''
-        Convert a tree structure to an IndexLevel instance.
+        Convert a tree structure to an IndexLevel instance. As a tree structure is a dictionary of keys to either a sequence of hashables or a dict of other keys, there is no way to represent a zero length, non-zero depth structure.
         '''
         # tree: tp.Dict[tp.Hashable, tp.Union[Sequence[tp.Hashable], tp.Dict]]
 
-        def get_index(labels: IndexInitializer, depth: int) -> IndexBase:
+        def get_index(labels: IndexInitializer, depth: int) -> Index:
             explicit_constructor: tp.Optional[IndexConstructor]
 
             if index_constructors is not None:
@@ -114,18 +116,32 @@ class IndexLevel:
             else:
                 explicit_constructor = None
 
-            return index_from_optional_constructor(labels,
+            return index_from_optional_constructor(labels, #type: ignore
                     default_constructor=cls._INDEX_CONSTRUCTOR,
                     explicit_constructor=explicit_constructor)
 
+        if len(tree) == 0:
+            return cls(get_index((), 0), depth_reference=depth_reference)
+        # NOTE: code check that returned object has depth equal depth_reference
         return cls.from_level_data(tree, get_index)
 
 
+    @classmethod
+    def from_depth(cls, depth: int) -> 'IndexLevel':
+        '''
+        Create zero-legnth IndexLevel from depth.
+        '''
+        return cls(cls._INDEX_CONSTRUCTOR(EMPTY_TUPLE),
+                own_index=True,
+                depth_reference=depth,
+                )
+
     def __init__(self,
             index: Index,
             targets: tp.Optional[ArrayGO] = None,
             offset: int = 0,
-            own_index: bool = False
+            own_index: bool = False,
+            depth_reference: tp.Optional[int] = None,
             ):
         '''
         Args:
@@ -133,11 +149,12 @@ class IndexLevel:
             offset: integer offset for this level.
             targets: None, or an ArrayGO of IndexLevel objects
             own_index: Boolean to determine whether the Index can be owned by this IndexLevel; if False, a static index will be reused if appropriate for this IndexLevel class.
+            depth_reference: for zero length Levels, provide the depth if it is greater than 1.
         '''
         if not isinstance(index, Index) or index.depth > 1:
-            # all derived Index should be depth == 1
             raise ErrorInitIndexLevel('cannot create an IndexLevel from a higher-dimensional Index.')
-        # NOTE: indices that conatain tuples will take additional work to support; we are not at this time checking for them, though values_at_depth will fail
+
+        # NOTE: indices that contain tuples will take additional work to support; we are not at this time checking for them, though values_at_depth will fail
 
         if own_index:
             self.index = index
@@ -146,8 +163,16 @@ class IndexLevel:
 
         self.targets = targets
         self.offset = offset
-        self._depth = None
-        self._length = None
+
+        # NOTE: once _depth is set, is ever re-evaluated over the life of the instance, even if it is an IndexLevelGO
+        if len(index) > 0:
+            self._depth = 1 if self.targets is None else None
+            self._length = None
+        else:
+            if depth_reference is None:
+                raise ErrorInitIndexLevel('zero length index requires specification of depth_reference')
+            self._depth = depth_reference
+            self._length = 0
 
     #---------------------------------------------------------------------------
     def depths(self) -> tp.Iterator[int]:
@@ -155,8 +180,9 @@ class IndexLevel:
         Get the depth of all leaves (which should all be the same). Mostly for integrity validation.
         '''
         # NOTE: as this uses a list instead of deque, the depths given will not be in the order of the actual leaves
-        if self.targets is None:
-            yield 1
+        if self.targets is None or not len(self.targets):
+            # NOTE: requires being set in __init__
+            yield self._depth #type: ignore
         else:
             levels = [(self, 0)]
             while levels:
@@ -169,13 +195,15 @@ class IndexLevel:
 
     def _get_depth(self) -> int:
         '''
-        Assuming all depths are uniform, can get the depth without storing levels list. Could store a depth attribute, but all nested components with provide overlapping depth descriptions that are never examined.
+        Called once over the life of an instance to set self._depth; this is not re-evaluated over the life of the instance.
         '''
-        if not len(self.index): # if zero sized, depth is zero
-            # TODO: need a way to represent 0-length IndexLevels with non-zero depth
-            return 1
-        if self.targets is None:
+        if not len(self.index):
+            raise AssertionError('zero-length indices should have depth set through depth_reference')
+
+        if self.targets is None: # this may not need to be here
             return 1
+
+        # if we need to recurse to the max depth
         level, depth = self, 1
         while True:
             if level.targets is None: # terminus
@@ -189,7 +217,8 @@ class IndexLevel:
         return self._depth
 
     def _get_length(self) -> int:
-        if self.targets is None:
+        # NOTE: unlike depth, length can change in an IndexLevelGO and should be re-evaluated.
+        if self.targets is None or not len(self.targets):
             return self.index.__len__()
 
         count = 0
@@ -206,6 +235,7 @@ class IndexLevel:
         '''
         The length is the sum of all leaves
         '''
+        # NOTE: in IndexLevelGO, setting length to None is how length is reset after mutation.
         if self._length is None:
             self._length = self._get_length()
         return self._length
@@ -273,7 +303,7 @@ class IndexLevel:
 
 
     def label_nodes_at_depth(self, depth_level: int) -> tp.Iterator[tp.Hashable]:
-        '''Given a depth position, iterate over label nodes at that depth. Only nodes will be provided, which for outer depths may not be of length equal to the entir index.
+        '''Given a depth position, iterate over label nodes at that depth. Only nodes will be provided, which for outer depths may not be of length equal to the entire index.
         '''
         if depth_level == 0:
             yield from self.index
@@ -292,7 +322,7 @@ class IndexLevel:
     # TODO: consider a different name; was dtypes()
     def dtypes_iter(self) -> tp.Iterator[np.dtype]:
         '''Return an iterator of all dtypes from every depth level.'''
-        if self.targets is None:
+        if self.targets is None or not len(self.targets):
             yield self.index.values.dtype
         else:
             levels = [self]
@@ -307,15 +337,21 @@ class IndexLevel:
         '''
         Return all dtypes found on a depth.
         '''
-        levels = deque(((self, 0),))
-        while levels:
-            level, depth = levels.popleft()
-            if depth == depth_level:
-                yield level.index.dtype
-                continue # do not need to descend
-            if level.targets is not None: # terminus
-                next_depth = depth + 1
-                levels.extend([(lvl, next_depth) for lvl in level.targets])
+        if not self.index.__len__():
+            if depth_level in range(self.depth):
+                yield self.index.dtype
+            else:
+                raise RuntimeError(f'invalid depth: {depth_level}')
+        else:
+            levels = deque(((self, 0),))
+            while levels:
+                level, depth = levels.popleft()
+                if depth == depth_level:
+                    yield level.index.dtype
+                    continue # do not need to descend
+                if level.targets is not None: # terminus
+                    next_depth = depth + 1
+                    levels.extend([(lvl, next_depth) for lvl in level.targets])
 
     def dtype_per_depth(self) -> tp.Iterator[np.dtype]:
         '''Return a tuple of resolved dtypes, one from each depth level.'''
@@ -325,8 +361,10 @@ class IndexLevel:
 
     # consider renaming index_types_per_depth
     def index_types(self) -> tp.Iterator[np.dtype]:
-        '''Return an iterator of reprsentative Index classes, one from each depth level.'''
-        if self.targets is None:
+        '''Return an iterator of representative Index classes, one from each depth level.'''
+        if not self.index.__len__():
+            yield from (self.index.__class__ for _ in range(self.depth))
+        elif self.targets is None:
             yield self.index.__class__
         else:
             levels = [self]
@@ -490,8 +528,8 @@ class IndexLevel:
         # need to get a compatible dtype for all dtypes
         dtype = resolve_dtype_iter(self.dtypes_iter())
         labels = np.empty(shape, dtype=dtype)
-        row_count = 0
 
+        row_count = 0
         levels = deque(((self, 0, None),)) # order matters
 
         while levels:
@@ -523,10 +561,6 @@ class IndexLevel:
         depth_count = self.depth
         levels = deque(((self, 0, None),)) # order matters
 
-        # level: IndexLevel
-        # depth: int
-        # row_previous: tp.Optional[tp.List[tp.Hashable]]
-
         while levels:
             level, depth, row_previous = levels.popleft()
 
@@ -545,9 +579,6 @@ class IndexLevel:
                     row[depth] = label
                     levels.append((level_target, depth_next, row)) #type: ignore
 
-    # def __iter__(self) -> tp.Iterator[tp.Tuple[tp.Hashable, ...]]:
-    #     part = [None] * self.depth
-    #     yield from _iter_recurse(self, part, 0)
 
     def values_at_depth(self,
             depth_level: int
@@ -557,10 +588,15 @@ class IndexLevel:
         '''
         depth_count = self.depth
         dtype = tuple(self.dtype_per_depth())[depth_level]
+
         length = self.__len__()
         # pre allocate array to ensure we use a resovled type
         array = np.empty(length, dtype=dtype)
 
+        if length == 0:
+            array.flags.writeable = False
+            return array
+
         if depth_level == depth_count - 1:
             # at maximal depth, can concat underlying arrays
             np.concatenate(
@@ -632,11 +668,12 @@ class IndexLevel:
                 skipna=skipna,
                 )
 
-        if self.targets is None and other.targets is None:
+        if ((self.targets is None or len(self.targets) == 0)
+                and (other.targets is None or len(other.targets) == 0)):
             return self.index.equals(other.index, **kwargs) #type: ignore
 
         # same length and depth, can traverse trees
-        # can store tuple of object ids to note those that have already been examine.
+        # can store tuple of object ids to note those that have already been examined.
         equal_pairs = set()
 
         levels_self = [self]
@@ -657,10 +694,10 @@ class IndexLevel:
             if level_self.targets is not None and level_other.targets is not None: # not terminus
                 levels_self.extend(level_self.targets)
                 levels_other.extend(level_other.targets)
-            if level_self.targets is None and level_other.targets is None: # not terminus
+            if level_self.targets is None and level_other.targets is None: # terminus
                 continue
-            if level_self.targets is None or level_other.targets is None: # not terminus
-                # at least one is at a terminus, but maybe both
+            if level_self.targets is None or level_other.targets is None:
+                # at least one is at a terminus
                 return False
 
         if not levels_self and not levels_other:
@@ -692,7 +729,11 @@ class IndexLevel:
             targets = None
 
         offset = self.offset if offset is None else offset
-        return cls(index=index, targets=targets, offset=offset) #type: ignore
+        return cls(index=index, #type: ignore
+                targets=targets,
+                offset=offset,
+                depth_reference=self.depth,
+                )
 
 
     def to_type_blocks(self) -> TypeBlocks:
@@ -763,16 +804,41 @@ class IndexLevelGO(IndexLevel):
         '''
         # find fist depth that does not contain key
         depth_count = self.depth
-        index_types = tuple(self.index_types())
 
         if len(key) != depth_count:
             raise RuntimeError('appending key {} of insufficent depth {}'.format(
                         key, depth_count))
 
+        if not self.index.__len__():
+            # where we have zero length, create new root index and targets from the key alone
+            depth_max = depth_count - 1
+            level_previous = None
+
+            for depth in range(depth_max, -1, -1):
+                k = key[depth]
+                # NOTE: we do not want to take index_types, as it based on notional index from zero length structure
+                # index_constructor = index_types[depth]
+                index = self._INDEX_CONSTRUCTOR((k,))
+                if depth == depth_max:
+                    targets = None
+                else:
+                    targets = ArrayGO([level_previous,], own_iterable=True)
+
+                if depth != 0:
+                    level_previous = IndexLevelGO(index, targets)
+                else:
+                    self.index = index
+                    self.targets = targets
+            self._length = None
+            return
+
+        # NOTE: does not use index_types when starting from a zero-length IndexLevels
+        index_types = tuple(self.index_types())
+
         depth_not_found = -1
         edge_nodes = np.empty(depth_count, dtype=object)
-
         node = self
+
         for depth, k in enumerate(key):
             edge_nodes[depth] = node
             # only set on first encounter in descent
