@@ -7,41 +7,36 @@ from itertools import chain
 
 import numpy as np
 
-from static_frame.core.frame import Frame
+
 from static_frame.core.bus import Bus
-
-from static_frame.core.util import DT64_S
-from static_frame.core.util import AnyCallable
-
+from static_frame.core.container import _RIGHT_OPERATOR_MAP
+from static_frame.core.container import _UFUNC_BINARY_OPERATORS
+from static_frame.core.container import _UFUNC_UNARY_OPERATORS
 from static_frame.core.container import ContainerBase
 from static_frame.core.container import ContainerOperand
-
-from static_frame.core.type_blocks import TypeBlocks
-from static_frame.core.index_base import IndexBase
-
-from static_frame.core.index_datetime import IndexDate
-from static_frame.core.index_datetime import IndexYearMonth
-from static_frame.core.index_datetime import IndexYear
-
-from static_frame.core.index_hierarchy import IndexHierarchy
 from static_frame.core.display import Display
+from static_frame.core.frame import Frame
 from static_frame.core.frame import FrameAsType
+from static_frame.core.index_base import IndexBase
+from static_frame.core.index_datetime import IndexDate
+from static_frame.core.index_datetime import IndexYear
+from static_frame.core.index_datetime import IndexYearMonth
+from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.node_dt import InterfaceDatetime
+from static_frame.core.node_iter import IterNodeDelegate
+from static_frame.core.node_selector import Interface
+from static_frame.core.node_selector import InterfaceAssignQuartet
+from static_frame.core.node_selector import InterfaceAssignTrio
+from static_frame.core.node_selector import InterfaceAsType
+from static_frame.core.node_selector import InterfaceGetItem
+from static_frame.core.node_selector import InterfaceSelectDuo
+from static_frame.core.node_selector import InterfaceSelectTrio
+from static_frame.core.node_selector import TContainer
+from static_frame.core.node_str import InterfaceString
+from static_frame.core.type_blocks import TypeBlocks
+from static_frame.core.util import AnyCallable
+from static_frame.core.util import DT64_S
 
-from static_frame.core.iter_node import IterNodeDelegate
-
-from static_frame.core.container import _UFUNC_BINARY_OPERATORS
-from static_frame.core.container import _RIGHT_OPERATOR_MAP
-from static_frame.core.container import _UFUNC_UNARY_OPERATORS
-
-from static_frame.core.selector_node import TContainer
-from static_frame.core.selector_node import Interface
-from static_frame.core.selector_node import InterfaceSelectDuo
-from static_frame.core.selector_node import InterfaceSelectTrio
-from static_frame.core.selector_node import InterfaceAssignTrio
-from static_frame.core.selector_node import InterfaceAssignQuartet
-
-from static_frame.core.selector_node import InterfaceAsType
-from static_frame.core.selector_node import InterfaceGetItem
 
 #-------------------------------------------------------------------------------
 # function inspection utilities
@@ -56,10 +51,10 @@ def _get_parameters(
     # might need special handling for methods on built-ins
     try:
         sig = inspect.signature(func)
-    except ValueError:
+    except ValueError: #pragma: no cover
         # on Python 3.6, this error happens:
         # ValueError: no signature found for builtin <built-in function abs>
-        return '[]' if is_getitem else '()'
+        return '[]' if is_getitem else '()' #pragma: no cover
 
     positional = []
     kwarg_only = ['*'] # preload
@@ -115,9 +110,11 @@ def _get_signatures(
     if delegate_func:
         delegate = _get_parameters(delegate_func, max_args=max_args)
         if delegate_name:
+            # prefix with name
             delegate = f'.{delegate_name}{delegate}'
-        # delegate is always assumed to not be a cls.getitem- style call sig
-        delegate_no_args = '()'
+            delegate_no_args = f'.{delegate_name}()'
+        else:
+            delegate_no_args = '()'
     else:
         delegate = ''
         delegate_no_args = ''
@@ -190,23 +187,6 @@ class Features:
         }
 
 
-    ATTR_ITER_NODE = (
-        'apply',
-        'apply_iter',
-        'apply_iter_items',
-        'apply_pool',
-        'map_all',
-        'map_all_iter',
-        'map_all_iter_items',
-        'map_any',
-        'map_any_iter',
-        'map_any_iter_items',
-        'map_fill',
-        'map_fill_iter',
-        'map_fill_iter_items',
-        )
-
-
     @classmethod
     def scrub_doc(cls, doc: tp.Optional[str]) -> str:
         if not doc:
@@ -236,6 +216,8 @@ class InterfaceGroup:
     OperatorUnary = 'Operator Unary'
     Selector = 'Selector'
     Assignment = 'Assignment'
+    AccessorString = 'Accessor String'
+    AccessorDatetime = 'Accessor Datetime'
 
 
 class InterfaceRecord(tp.NamedTuple):
@@ -252,7 +234,7 @@ class InterfaceRecord(tp.NamedTuple):
     signature_no_args: str = ''
 
     @classmethod
-    def from_dict_like(cls, *,
+    def gen_from_dict_like(cls, *,
             cls_name: str,
             name: str,
             obj: AnyCallable,
@@ -263,7 +245,7 @@ class InterfaceRecord(tp.NamedTuple):
         if name == 'values':
             signature = signature_no_args = name
         else:
-            signature, signature_no_arg_get_signatures = _get_signatures(
+            signature, signature_no_args = _get_signatures(
                     name,
                     obj,
                     is_getitem=False,
@@ -274,11 +256,11 @@ class InterfaceRecord(tp.NamedTuple):
                 signature,
                 doc,
                 reference,
-                signature_no_args=signature
+                signature_no_args=signature_no_args
                 )
 
     @classmethod
-    def from_display(cls, *,
+    def gen_from_display(cls, *,
             cls_name: str,
             name: str,
             obj: AnyCallable,
@@ -311,7 +293,7 @@ class InterfaceRecord(tp.NamedTuple):
                     )
 
     @classmethod
-    def from_astype(cls, *,
+    def gen_from_astype(cls, *,
             cls_name: str,
             name: str,
             obj: AnyCallable,
@@ -365,7 +347,7 @@ class InterfaceRecord(tp.NamedTuple):
 
 
     @classmethod
-    def from_constructor(cls, *,
+    def gen_from_constructor(cls, *,
             cls_name: str,
             name: str,
             obj: AnyCallable,
@@ -389,7 +371,7 @@ class InterfaceRecord(tp.NamedTuple):
                 )
 
     @classmethod
-    def from_exporter(cls, *,
+    def gen_from_exporter(cls, *,
             cls_name: str,
             name: str,
             obj: AnyCallable,
@@ -413,7 +395,7 @@ class InterfaceRecord(tp.NamedTuple):
                 )
 
     @classmethod
-    def from_iterator(cls, *,
+    def gen_from_iterator(cls, *,
             cls_name: str,
             name: str,
             obj: AnyCallable,
@@ -422,7 +404,6 @@ class InterfaceRecord(tp.NamedTuple):
             max_args: int,
             ) -> tp.Iterator['InterfaceRecord']:
 
-        is_attr = True
         signature, signature_no_args = _get_signatures(
                 name,
                 obj.__call__, #type: ignore
@@ -440,8 +421,7 @@ class InterfaceRecord(tp.NamedTuple):
                 signature_no_args=signature_no_args,
                 )
 
-        for field in Features.ATTR_ITER_NODE: # apply, map, etc
-
+        for field in IterNodeDelegate.INTERFACE: # apply, map, etc
             delegate_obj = getattr(IterNodeDelegate, field)
             delegate_reference = f'{IterNodeDelegate.__name__}.{field}'
             doc = Features.scrub_doc(delegate_obj.__doc__)
@@ -465,6 +445,58 @@ class InterfaceRecord(tp.NamedTuple):
                     signature_no_args=signature_no_args
                     )
 
+    @classmethod
+    def gen_from_accessor(cls, *,
+            cls_name: str,
+            name: str,
+            obj: AnyCallable,
+            reference: str,
+            doc: str,
+            cls_interface: tp.Type[Interface[TContainer]],
+            max_args: int,
+            ) -> tp.Iterator['InterfaceRecord']:
+
+        group = (InterfaceGroup.AccessorString
+                if cls_interface is InterfaceString
+                else InterfaceGroup.AccessorDatetime)
+
+        for field in cls_interface.INTERFACE: # apply, map, etc
+            delegate_obj = getattr(cls_interface, field)
+            delegate_reference = f'{cls_interface.__name__}.{field}'
+            doc = Features.scrub_doc(delegate_obj.__doc__)
+
+            terminus_name = f'{name}.{field}'
+
+            if isinstance(delegate_obj, property):
+                # some date tools are properties
+                yield InterfaceRecord(cls_name,
+                        group,
+                        terminus_name,
+                        doc,
+                        reference,
+                        is_attr=True,
+                        use_signature=True,
+                        delegate_reference=delegate_reference,
+                        delegate_is_attr=True,
+                        signature_no_args=terminus_name
+                        )
+            else:
+                signature, signature_no_args = _get_signatures(
+                        terminus_name,
+                        delegate_obj,
+                        max_args=max_args,
+                        )
+                yield cls(cls_name,
+                        group,
+                        signature,
+                        doc,
+                        reference,
+                        is_attr=True,
+                        use_signature=True,
+                        delegate_reference=delegate_reference,
+                        signature_no_args=signature_no_args
+                        )
+
     @classmethod
     def from_getitem(cls, *,
             cls_name: str,
@@ -502,7 +534,7 @@ class InterfaceRecord(tp.NamedTuple):
 
 
     @classmethod
-    def from_selection(cls, *,
+    def gen_from_selection(cls, *,
             cls_name: str,
             name: str,
             obj: AnyCallable,
@@ -549,7 +581,7 @@ class InterfaceRecord(tp.NamedTuple):
 
 
     @classmethod
-    def from_assignment(cls, *,
+    def gen_from_assignment(cls, *,
             cls_name: str,
             name: str,
             obj: tp.Union[InterfaceAssignTrio[TContainer],
@@ -598,13 +630,13 @@ class InterfaceRecord(tp.NamedTuple):
                     terminus_doc,
                     reference,
                     use_signature=True,
-                    is_attr=False,
+                    is_attr=True,
                     delegate_reference=terminus_reference,
                     signature_no_args=signature_no_args
                     )
 
     @classmethod
-    def from_method(cls, *,
+    def gen_from_method(cls, *,
             cls_name: str,
             name: str,
             obj: AnyCallable,
@@ -747,29 +779,39 @@ class InterfaceSummary(Features):
                     )
 
             if name in cls.DICT_LIKE:
-                yield from InterfaceRecord.from_dict_like(**kwargs)
+                yield from InterfaceRecord.gen_from_dict_like(**kwargs)
             elif name in cls.DISPLAY:
-                yield from InterfaceRecord.from_display(**kwargs)
+                yield from InterfaceRecord.gen_from_display(**kwargs)
             elif name == 'astype':
-                yield from InterfaceRecord.from_astype(**kwargs)
+                yield from InterfaceRecord.gen_from_astype(**kwargs)
             elif name.startswith('from_') or name == '__init__':
-                yield from InterfaceRecord.from_constructor(**kwargs)
+                yield from InterfaceRecord.gen_from_constructor(**kwargs)
             elif name.startswith('to_'):
-                yield from InterfaceRecord.from_exporter(**kwargs)
+                yield from InterfaceRecord.gen_from_exporter(**kwargs)
             elif name.startswith('iter_'):
-                yield from InterfaceRecord.from_iterator(**kwargs)
+                yield from InterfaceRecord.gen_from_iterator(**kwargs)
             elif isinstance(obj, InterfaceGetItem) or name == cls.GETITEM:
                 yield from InterfaceRecord.from_getitem(**kwargs)
+            elif isinstance(obj, InterfaceString):
+                yield from InterfaceRecord.gen_from_accessor(
+                            cls_interface=InterfaceString,
+                            **kwargs,
+                            )
+            elif isinstance(obj, InterfaceDatetime):
+                yield from InterfaceRecord.gen_from_accessor(
+                            cls_interface=InterfaceDatetime,
+                            **kwargs,
+                            )
             elif obj.__class__ in (InterfaceSelectDuo, InterfaceSelectTrio):
-                yield from InterfaceRecord.from_selection(
+                yield from InterfaceRecord.gen_from_selection(
                         cls_interface=obj.__class__,
                         **kwargs)
             elif obj.__class__ in (InterfaceAssignTrio, InterfaceAssignQuartet):
-                yield from InterfaceRecord.from_assignment(
+                yield from InterfaceRecord.gen_from_assignment(
                         cls_interface=obj.__class__,
                         **kwargs)
             elif callable(obj): # general methods
-                yield from InterfaceRecord.from_method(**kwargs)
+                yield from InterfaceRecord.gen_from_method(**kwargs)
             else: # attributes
                 yield InterfaceRecord(cls_name,
                         InterfaceGroup.Attribute,
