@@ -8,7 +8,8 @@ from functools import reduce
 from io import StringIO
 from itertools import chain
 from itertools import zip_longest
-from pathlib import Path
+# from pathlib import Path
+from os import PathLike
 from urllib import request
 import datetime
 import operator
@@ -53,13 +54,17 @@ DEFAULT_STABLE_SORT_KIND = 'mergesort'
 # ARCHITECTURE_SIZE = struct.calcsize('P') * 8 # size of pointer
 # ARCHITECTURE_INT_DTYPE = np.int64 if ARCHITECTURE_SIZE == 64 else np.int32
 
-DTYPE_STR_KIND = ('U', 'S') # S is np.bytes_
-DTYPE_INT_KIND = ('i', 'u') # signed and unsigned
-DTYPE_NAN_KIND = ('f', 'c') # kinds that support NaN values
+DTYPE_STR_KINDS = ('U', 'S') # S is np.bytes_
+DTYPE_INT_KINDS = ('i', 'u') # signed and unsigned
+DTYPE_INEXACT_KINDS = ('f', 'c') # kinds that support NaN values
+DTYPE_NAT_KINDS = ('M', 'm')
+
 DTYPE_DATETIME_KIND = 'M'
 DTYPE_TIMEDELTA_KIND = 'm'
 DTYPE_COMPLEX_KIND = 'c'
-DTYPE_NAT_KIND = ('M', 'm')
+DTYPE_FLOAT_KIND = 'f'
+DTYPE_OBJECT_KIND = 'O'
+
 # DTYPE_BOOL_KIND = ('b',)
 
 DTYPE_OBJECT = np.dtype(object)
@@ -110,7 +115,7 @@ TIME_DELTA_ATTR_MAP = (
         ('microseconds', 'us')
         )
 
-# ufunc functions that will not work with DTYPE_STR_KIND, but do work if converted to object arrays; see UFUNC_AXIS_SKIPNA for the matching functions
+# ufunc functions that will not work with DTYPE_STR_KINDS, but do work if converted to object arrays; see UFUNC_AXIS_SKIPNA for the matching functions
 UFUNC_AXIS_STR_TO_OBJ = {np.min, np.max, np.sum}
 
 #-------------------------------------------------------------------------------
@@ -120,6 +125,7 @@ INT_TYPES = (int, np.integer) # np.integer catches all np int
 FLOAT_TYPES = (float, np.floating) # np.floating catches all np float
 COMPLEX_TYPES = (complex, np.complexfloating) # np.complexfloating catches all np complex
 INEXACT_TYPES = (float, complex, np.inexact) # inexact matches floating, complexfloating
+InexactTypes = tp.Union[float, complex, np.inexact]
 NUMERIC_TYPES = (int, float, complex, np.number)
 
 BOOL_TYPES = (bool, np.bool_)
@@ -186,9 +192,9 @@ CallableOrCallableMap = tp.Union[AnyCallable, tp.Mapping[tp.Hashable, AnyCallabl
 # for explivitl selection hashables, or things that will be converted to lists of hashables (explicitly lists)
 KeyOrKeys = tp.Union[tp.Hashable, tp.Iterable[tp.Hashable]]
 
-PathSpecifier = tp.Union[str, Path]
-PathSpecifierOrFileLike = tp.Union[str, Path, tp.TextIO]
-PathSpecifierOrFileLikeOrIterator = tp.Union[str, Path, tp.TextIO, tp.Iterator[str]]
+PathSpecifier = tp.Union[str, PathLike]
+PathSpecifierOrFileLike = tp.Union[str, PathLike, tp.TextIO]
+PathSpecifierOrFileLikeOrIterator = tp.Union[str, PathLike, tp.TextIO, tp.Iterator[str]]
 
 
 DtypeSpecifier = tp.Optional[tp.Union[str, np.dtype, type]]
@@ -383,8 +389,8 @@ def resolve_dtype(dt1: np.dtype, dt2: np.dtype) -> np.dtype:
     if dt1.kind == 'O' or dt2.kind == 'O':
         return DTYPE_OBJECT
 
-    dt1_is_str = dt1.kind in DTYPE_STR_KIND
-    dt2_is_str = dt2.kind in DTYPE_STR_KIND
+    dt1_is_str = dt1.kind in DTYPE_STR_KINDS
+    dt2_is_str = dt2.kind in DTYPE_STR_KINDS
     if dt1_is_str and dt2_is_str:
         # if both are string or string-like, we can use result type to get the longest string
         return np.result_type(dt1, dt2)
@@ -491,15 +497,15 @@ def dtype_to_na(dtype: DtypeSpecifier) -> tp.Any:
 
     kind = dtype.kind
 
-    if kind in DTYPE_INT_KIND:
+    if kind in DTYPE_INT_KINDS:
         return 0 # cannot support NaN
     elif kind == 'b':
         return False
-    elif kind in DTYPE_NAN_KIND:
+    elif kind in DTYPE_INEXACT_KINDS:
         return np.nan
     elif kind == 'O':
         return None
-    elif kind in DTYPE_STR_KIND:
+    elif kind in DTYPE_STR_KINDS:
         return ''
     elif kind in DTYPE_DATETIME_KIND:
         return NAT
@@ -545,7 +551,7 @@ def ufunc_axis_skipna(
         # dates do not support skipna functions
         return ufunc(array, axis=axis, out=out)
 
-    elif array.dtype.kind in DTYPE_STR_KIND and ufunc in UFUNC_AXIS_STR_TO_OBJ:
+    elif array.dtype.kind in DTYPE_STR_KINDS and ufunc in UFUNC_AXIS_STR_TO_OBJ:
         v = array.astype(object)
     else:
         v = array
@@ -1154,9 +1160,9 @@ def isna_array(array: np.ndarray,
     '''
     kind = array.dtype.kind
     # matches all floating point types
-    if kind in DTYPE_NAN_KIND:
+    if kind in DTYPE_INEXACT_KINDS:
         return np.isnan(array)
-    elif kind in DTYPE_NAT_KIND:
+    elif kind in DTYPE_NAT_KINDS:
         return np.isnat(array)
     # match everything that is not an object; options are: biufcmMOSUV
     elif kind != 'O':
@@ -1502,8 +1508,8 @@ def _ufunc_set_1d(
                     return array
 
     set_compare = False
-    array_is_str = array.dtype.kind in DTYPE_STR_KIND
-    other_is_str = other.dtype.kind in DTYPE_STR_KIND
+    array_is_str = array.dtype.kind in DTYPE_STR_KINDS
+    other_is_str = other.dtype.kind in DTYPE_STR_KINDS
 
     if array_is_str ^ other_is_str:
         # if only one is string
@@ -1771,8 +1777,9 @@ def ufunc_set_iter(
 
         if not union and len(result) == 0:
             # short circuit intersection that results in no common values
-            return result
+            break
 
+    result.flags.writeable = False
     return result
 
 
@@ -2022,7 +2029,7 @@ def slices_from_targets(
 def path_filter(fp: PathSpecifierOrFileLike) -> tp.Union[str, tp.TextIO]:
     '''Realize Path objects as strings, let TextIO pass through, if given.
     '''
-    if isinstance(fp, Path):
+    if isinstance(fp, PathLike):
         return str(fp)
     return fp
 
